'use strict';
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var services_1 = require("@jupyterlab/services");
var apputils_1 = require("@jupyterlab/apputils");
var semver = require("semver");
var query_1 = require("./query");
/**
 * The server API path for querying/modifying installed extensions.
 */
var EXTENSION_API_PATH = "lab/api/extensions";
/**
 * Model for an extension list.
 */
var ListModel = /** @class */ (function (_super) {
    __extends(ListModel, _super);
    function ListModel() {
        var _this = _super.call(this) || this;
        /**
         * Whether the model has finished async initialization.
         */
        _this.initialized = false;
        _this._query = '';
        _this._page = 0;
        _this._pagination = 250;
        _this._totalEntries = 0;
        /**
         * A helper for performing searches of jupyterlab extensions on the NPM repository.
         */
        _this.searcher = new query_1.Searcher();
        _this._installed = [];
        _this._searchResult = [];
        _this.serverConnectionSettings = services_1.ServerConnection.makeSettings({ withCredentials: true });
        return _this;
    }
    /**
     * Utility function to check whether an entry can be updated.
     *
     * @param entry The entry to check.
     */
    ListModel.entryHasUpdate = function (entry) {
        if (!entry.installed || !entry.latest_version) {
            return false;
        }
        return semver.lt(entry.installed_version, entry.latest_version);
    };
    Object.defineProperty(ListModel.prototype, "installed", {
        /**
         * A readonly array of the installed extensions.
         */
        get: function () {
            return this._installed;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListModel.prototype, "searchResult", {
        /**
         * A readonly array containing the latest search result
         */
        get: function () {
            return this._searchResult;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Translate search results from an npm repository query into entries.
     *
     * @param res Promise to an npm query result.
     */
    ListModel.prototype.translateSearchResult = function (res) {
        return __awaiter(this, void 0, void 0, function () {
            var entries, _i, _a, obj, pkg;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        entries = {};
                        _i = 0;
                        return [4 /*yield*/, res];
                    case 1:
                        _a = (_b.sent()).objects;
                        _b.label = 2;
                    case 2:
                        if (!(_i < _a.length)) return [3 /*break*/, 4];
                        obj = _a[_i];
                        pkg = obj.package;
                        entries[pkg.name] = {
                            name: pkg.name,
                            description: pkg.description,
                            python_package: null,
                            installed: false,
                            enabled: false,
                            status: null,
                            latest_version: pkg.version,
                            installed_version: '',
                        };
                        _b.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 2];
                    case 4: return [2 /*return*/, entries];
                }
            });
        });
    };
    /**
     * Translate installed extensions information from the server into entries.
     *
     * @param res Promise to the server reply data.
     */
    ListModel.prototype.translateInstalled = function (res) {
        return __awaiter(this, void 0, void 0, function () {
            var promises, entries, _loop_1, _i, _a, pkg;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        promises = [];
                        entries = {};
                        _loop_1 = function (pkg) {
                            promises.push(res.then(function (info) {
                                entries[pkg.name] = {
                                    name: pkg.name,
                                    description: pkg.description,
                                    python_package: null,
                                    installed: true,
                                    enabled: pkg.enabled,
                                    status: pkg.status,
                                    latest_version: pkg.latest_version,
                                    installed_version: pkg.installed_version,
                                };
                            }));
                        };
                        _i = 0;
                        return [4 /*yield*/, res];
                    case 1:
                        _a = _b.sent();
                        _b.label = 2;
                    case 2:
                        if (!(_i < _a.length)) return [3 /*break*/, 4];
                        pkg = _a[_i];
                        _loop_1(pkg);
                        _b.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 2];
                    case 4: return [2 /*return*/, Promise.all(promises).then(function () {
                            return entries;
                        })];
                }
            });
        });
    };
    Object.defineProperty(ListModel.prototype, "query", {
        /**
         * The current NPM repository search query.
         *
         * Setting its value triggers a new search.
         */
        get: function () {
            return this._query;
        },
        set: function (value) {
            this._query = value;
            this.update();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListModel.prototype, "page", {
        /**
         * The current NPM repository search page.
         *
         * The npm repository search is paginated by the `pagination` attribute.
         * The `page` value selects which page is used.
         *
         * Setting its value triggers a new search.
         */
        get: function () {
            return this._page;
        },
        set: function (value) {
            this._page = value;
            this.update();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListModel.prototype, "pagination", {
        /**
         * The NPM repository search pagination.
         *
         * The npm repository search is paginated by the `pagination` attribute.
         * The `page` value selects which page is used.
         *
         * Setting its value triggers a new search.
         */
        get: function () {
            return this._pagination;
        },
        set: function (value) {
            this._pagination = value;
            this.update();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListModel.prototype, "totalEntries", {
        /**
         * The total number of results in the current search.
         */
        get: function () {
            return this._totalEntries;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Make a request to the server for info about its installed extensions.
     */
    ListModel.prototype.fetchInstalled = function () {
        var request = {
            url: EXTENSION_API_PATH,
        };
        return services_1.ServerConnection.makeRequest(request, this.serverConnectionSettings).then(function (response) {
            return response.data;
        });
    };
    /**
     * Initialize the model.
     */
    ListModel.prototype.initialize = function () {
        var _this = this;
        this.update().then(function () {
            _this.initialized = true;
            _this.stateChanged.emit(undefined);
        }).catch(function () {
            _this.initialized = true;
            _this.stateChanged.emit(undefined);
        });
    };
    /**
     * Update the current model.
     *
     * This will query the NPM repository, and the notebook server.
     *
     * Emits the `stateChanged` signal on succesfull completion.
     */
    ListModel.prototype.update = function () {
        return __awaiter(this, void 0, void 0, function () {
            var search, searchMapPromise, installedMap, searchMap, reason_1, installed, _i, _a, key, searchResult, _b, _c, key, _d, error_1;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        search = this.searcher.searchExtensions(this.query, this.page, this.pagination);
                        searchMapPromise = this.translateSearchResult(search);
                        return [4 /*yield*/, this.translateInstalled(this.fetchInstalled())];
                    case 1:
                        installedMap = _e.sent();
                        _e.label = 2;
                    case 2:
                        _e.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, searchMapPromise];
                    case 3:
                        searchMap = _e.sent();
                        this.offline = false;
                        this.errorMessage = undefined;
                        return [3 /*break*/, 5];
                    case 4:
                        reason_1 = _e.sent();
                        searchMap = {};
                        this.offline = true;
                        this.errorMessage = reason_1.toString();
                        return [3 /*break*/, 5];
                    case 5:
                        installed = [];
                        for (_i = 0, _a = Object.keys(installedMap); _i < _a.length; _i++) {
                            key = _a[_i];
                            installed.push(installedMap[key]);
                        }
                        this._installed = installed;
                        searchResult = [];
                        for (_b = 0, _c = Object.keys(searchMap); _b < _c.length; _b++) {
                            key = _c[_b];
                            if (installedMap[key] === undefined) {
                                searchResult.push(searchMap[key]);
                            }
                            else {
                                searchResult.push(installedMap[key]);
                            }
                        }
                        this._searchResult = searchResult;
                        _e.label = 6;
                    case 6:
                        _e.trys.push([6, 8, , 9]);
                        _d = this;
                        return [4 /*yield*/, search];
                    case 7:
                        _d._totalEntries = (_e.sent()).total;
                        return [3 /*break*/, 9];
                    case 8:
                        error_1 = _e.sent();
                        this._totalEntries = 0;
                        return [3 /*break*/, 9];
                    case 9:
                        this.stateChanged.emit(undefined);
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Send a request to the server to perform an action on an extension.
     *
     * @param action A valid action to perform.
     * @param entry The extension to perform the action on.
     */
    ListModel.prototype._performAction = function (action, entry) {
        var request = {
            url: EXTENSION_API_PATH,
            method: 'POST',
            data: JSON.stringify({
                cmd: action,
                extension_name: entry.name,
            }),
        };
        return services_1.ServerConnection.makeRequest(request, this.serverConnectionSettings).then(function (response) {
            return response.data;
        });
    };
    /**
     * Install an extension.
     *
     * @param entry An entry indicating which extension to install.
     */
    ListModel.prototype.install = function (entry) {
        var _this = this;
        if (entry.installed) {
            throw new Error("Already installed: " + entry.name);
        }
        this._performAction('install', entry).then(function (data) {
            _this.update();
        });
    };
    /**
     * Uninstall an extension.
     *
     * @param entry An entry indicating which extension to uninstall.
     */
    ListModel.prototype.uninstall = function (entry) {
        var _this = this;
        if (!entry.installed) {
            throw new Error("Not installed, cannot uninstall: " + entry.name);
        }
        this._performAction('uninstall', entry).then(function (data) {
            _this.update();
        });
        ;
    };
    /**
     * Enable an extension.
     *
     * @param entry An entry indicating which extension to enable.
     */
    ListModel.prototype.enable = function (entry) {
        var _this = this;
        if (entry.enabled) {
            throw new Error("Already enabled: " + entry.name);
        }
        this._performAction('enable', entry).then(function (data) {
            _this.update();
        });
        ;
    };
    /**
     * Disable an extension.
     *
     * @param entry An entry indicating which extension to disable.
     */
    ListModel.prototype.disable = function (entry) {
        var _this = this;
        if (!entry.enabled) {
            throw new Error("Already disabled: " + entry.name);
        }
        this._performAction('disable', entry).then(function (data) {
            _this.update();
        });
        ;
    };
    return ListModel;
}(apputils_1.VDomModel));
exports.ListModel = ListModel;
//# sourceMappingURL=model.js.map