'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var application_1 = require("@jupyterlab/application");
var widget_1 = require("./widget");
require("../style/index.css");
/**
 * The discover-view namespace token.
 */
var namespaceToken = 'discoverview';
/**
 * IDs of the commands added by this extension.
 */
var CommandIDs;
(function (CommandIDs) {
    CommandIDs.hideDiscover = 'discover:hide-main';
    CommandIDs.showDiscover = 'discover:activate-main';
    CommandIDs.toggleDiscover = 'discover:toggle-main';
})(CommandIDs || (CommandIDs = {}));
/**
 * Initialization data for the jupyterlab_discovery extension.
 */
var extension = {
    id: 'jupyterlab_discovery',
    autoStart: true,
    requires: [application_1.ILayoutRestorer],
    activate: function (app, restorer) {
        var commands = app.commands, shell = app.shell;
        var view = new widget_1.ExtensionView();
        view.id = 'discover.main-view';
        restorer.add(view, namespaceToken);
        view.title.label = 'Extensions';
        shell.addToLeftArea(view, { rank: 100 });
        // If the layout is a fresh session without saved data, open file view.
        app.restored.then(function (layout) {
            if (layout.fresh) {
                commands.execute(CommandIDs.showDiscover, void 0);
            }
        });
        addCommands(app, view);
    }
};
/**
 * Add the main file view commands to the application's command registry.
 */
function addCommands(app, view) {
    var commands = app.commands;
    commands.addCommand(CommandIDs.showDiscover, {
        label: 'Discover extensions',
        caption: 'Show view to discover extensions',
        execute: function () { app.shell.activateById(view.id); }
    });
    commands.addCommand(CommandIDs.hideDiscover, {
        execute: function () {
            if (!view.isHidden) {
                app.shell.collapseLeft();
            }
        }
    });
    commands.addCommand(CommandIDs.toggleDiscover, {
        execute: function () {
            if (view.isHidden) {
                return commands.execute(CommandIDs.showDiscover, void 0);
            }
            else {
                return commands.execute(CommandIDs.hideDiscover, void 0);
            }
        }
    });
    // TODO: Also add to command palette
}
exports.default = extension;
//# sourceMappingURL=plugin.js.map