/**
 * Information about a person in search results.
 */
export interface IPerson {
    username: string;
    email: string;
}
/**
 * NPM registry search result structure.
 */
export interface ISearchResult {
    objects: {
        package: {
            name: string;
            scope: string;
            version: string;
            description: string;
            keywords: string[];
            date: string;
            links: {
                [key: string]: string;
            };
            publisher: IPerson;
            maintainers: IPerson[];
        };
        flags: {
            insecure: number;
            unstable: boolean;
        };
        score: {
            final: number;
            detail: {
                quality: number;
                popularity: number;
                maintenance: number;
            };
        };
        searchScore: number;
    }[];
    total: number;
    time: string;
}
/**
 * An interface for a subset of the keys known to be included for package metadata.
 *
 * See https://github.com/npm/registry/blob/master/docs/responses/package-metadata.md
 * for full specification.
 */
export interface IPackageMetadata {
    name: string;
    modified: string;
    "dist-tags": {
        latest: string;
        [key: string]: string;
    };
    description: string;
    versions: {
        [key: string]: {
            name: string;
            version: string;
            deprecated?: string;
            description: string;
        };
    };
}
/**
 * Searches the NPM registry via web API: https://github.com/npm/registry/blob/master/docs/REGISTRY-API.md
 */
export declare class Searcher {
    constructor(repoUri?: string);
    /**
     * Search for a jupyterlab extension.
     *
     * @param query The query to send. `keywords:"jupyterlab extension"` will be appended to the query.
     * @param page The page of results to fetch.
     * @param pageination The pagination size to use. See registry API documentation for acceptable values.
     */
    searchExtensions(query: string, page?: number, pageination?: number): Promise<ISearchResult>;
    repoUri: string;
}
