import * as React from 'react';
import { VDomRenderer } from '@jupyterlab/apputils';
import { Message } from '@phosphor/messaging';
import { ListModel, IEntry, Action } from './model';
/**
 * Search bar VDOM component.
 */
export declare class SearchBar extends React.Component<SearchBar.IProperties, SearchBar.IState> {
    constructor(props: SearchBar.IProperties);
    /**
     * Render the list view using the virtual DOM.
     */
    render(): React.ReactElement<any>;
    /**
     * Handler for search input changes.
     */
    handleChange(e: KeyboardEvent): void;
}
export declare namespace SearchBar {
    interface IProperties {
        /**
         * The placeholder string to use in the search bar input field when empty.
         */
        placeholder: string;
    }
    interface IState {
        /**
         * The value of the search bar input field.
         */
        value: string;
    }
}
export declare namespace ListEntry {
    interface IProperties {
        /**
         * The entry to visualize.
         */
        entry: IEntry;
        /**
         * Callback to use for performing an action on the entry.
         */
        performAction: (action: Action, entry: IEntry) => void;
    }
}
/**
 * List view widget for extensions
 */
export declare function ListView(props: ListView.IProperties): React.ReactElement<any>;
export declare namespace ListView {
    interface IProperties {
        /**
         * The extension entries to display.
         */
        entries: ReadonlyArray<IEntry>;
        /**
         * The number of pages that can be viewed via pagination.
         */
        numPages: number;
        /**
         * The callback to use for changing the page
         */
        onPage: (page: number) => void;
        /**
         * Callback to use for performing an action on an entry.
         */
        performAction: (action: Action, entry: IEntry) => void;
    }
}
/**
 * The main view for the discovery extension.
 */
export declare class ExtensionView extends VDomRenderer<ListModel> {
    constructor();
    /**
     * The search input node.
     */
    readonly inputNode: HTMLInputElement;
    /**
     * Render the extension discovery view using the virtual DOM.
     */
    protected render(): React.ReactElement<any>[];
    /**
     * Callback handler for the user specifies a new search query.
     *
     * @param value The new query.
     */
    onSearch(value: string): void;
    /**
     * Callback handler for the user changes the page of the search result pagination.
     *
     * @param value The pagination page number.
     */
    onPage(value: number): void;
    /**
     * Callback handler for when the user wants to perform an action on an extension.
     *
     * @param action The action to perform.
     * @param entry The entry to perform the action on.
     */
    onAction(action: Action, entry: IEntry): void;
    /**
   * Handle the DOM events for the command palette.
   *
   * @param event - The DOM event sent to the command palette.
   *
   * #### Notes
   * This method implements the DOM `EventListener` interface and is
   * called in response to events on the command palette's DOM node.
   * It should not be called directly by user code.
   */
    handleEvent(event: Event): void;
    /**
     * A message handler invoked on a `'before-attach'` message.
     */
    protected onBeforeAttach(msg: Message): void;
    /**
     * A message handler invoked on an `'after-detach'` message.
     */
    protected onAfterDetach(msg: Message): void;
    /**
     * A message handler invoked on an `'activate-request'` message.
     */
    protected onActivateRequest(msg: Message): void;
    /**
     * Toggle the focused modifier based on the input node focus state.
     */
    private _toggleFocused();
}
