'use strict';
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var apputils_1 = require("@jupyterlab/apputils");
var ReactPaginate = require("react-paginate");
var model_1 = require("./model");
// TODO: Replace pagination with lazy loading of lower search results
/**
 * Search bar VDOM component.
 */
var SearchBar = /** @class */ (function (_super) {
    __extends(SearchBar, _super);
    function SearchBar(props) {
        var _this = _super.call(this, props) || this;
        _this.state = {
            value: '',
        };
        return _this;
    }
    /**
     * Render the list view using the virtual DOM.
     */
    SearchBar.prototype.render = function () {
        return (React.createElement("div", { className: 'jp-discovery-search-bar' },
            React.createElement("div", { className: 'jp-discovery-search-wrapper' },
                React.createElement("input", { type: 'text', className: 'jp-discovery-input', placeholder: this.props.placeholder, onChange: this.handleChange.bind(this), value: this.state.value }))));
    };
    /**
     * Handler for search input changes.
     */
    SearchBar.prototype.handleChange = function (e) {
        var target = e.target;
        this.setState({
            value: target.value,
        });
    };
    return SearchBar;
}(React.Component));
exports.SearchBar = SearchBar;
/**
 * VDOM for visualizing an extension entry.
 */
function ListEntry(props) {
    var entry = props.entry;
    var flagClasses = [];
    if (entry.python_package) {
        flagClasses.push('jp-discovery-entry-pure');
    }
    if (entry.installed) {
        flagClasses.push('jp-discovery-entry-installed');
    }
    if (entry.enabled) {
        flagClasses.push('jp-discovery-entry-enabled');
    }
    if (model_1.ListModel.entryHasUpdate(entry)) {
        flagClasses.push('jp-discovery-entry-update');
    }
    if (entry.status && ['ok', 'warning', 'error'].indexOf(entry.status) !== -1) {
        flagClasses.push("jp-discovery-entry-" + entry.status);
    }
    return (React.createElement("li", { className: "jp-discovery-entry " + flagClasses.join(' ') },
        React.createElement("div", { className: 'jp-discovery-entry-name' }, entry.name),
        React.createElement("div", { className: 'jp-discovery-entry-content' },
            React.createElement("div", { className: 'jp-discovery-entry-description' }, entry.description),
            React.createElement("div", { className: 'jp-discovery-entry-buttons' },
                React.createElement("button", { className: 'jp-discovery-install', onClick: function () { return props.performAction('install', entry); } }, "Install"),
                React.createElement("button", { className: 'jp-discovery-update', 
                    // An install action will update the extension:
                    onClick: function () { return props.performAction('install', entry); } }, "Update"),
                React.createElement("button", { className: 'jp-discovery-uninstall', onClick: function () { return props.performAction('uninstall', entry); } }, "Uninstall"),
                React.createElement("button", { className: 'jp-discovery-enable', onClick: function () { return props.performAction('enable', entry); } }, "Enable"),
                React.createElement("button", { className: 'jp-discovery-disable', onClick: function () { return props.performAction('disable', entry); } }, "Disable")))));
}
/**
 * List view widget for extensions
 */
function ListView(props) {
    var entryViews = [];
    for (var _i = 0, _a = props.entries; _i < _a.length; _i++) {
        var entry = _a[_i];
        entryViews.push(React.createElement(ListEntry, { entry: entry, key: entry.name, performAction: props.performAction }));
    }
    var pagination;
    if (props.numPages > 1) {
        pagination = (React.createElement("div", { className: 'jp-discovery-pagination' },
            React.createElement(ReactPaginate, { previousLabel: "<", nextLabel: ">", breakLabel: React.createElement("a", { href: "" }, "..."), breakClassName: "break-me", pageCount: props.numPages, marginPagesDisplayed: 2, pageRangeDisplayed: 5, onPageChange: function (data) { return props.onPage(data.selected); }, containerClassName: "pagination", activeClassName: "active" })));
    }
    var listview = (React.createElement("ul", { className: 'jp-discovery-listview' }, entryViews));
    return (React.createElement("div", { className: 'jp-discovery-listview-wrapper' },
        entryViews.length > 0 ? listview : React.createElement("div", { key: "message", className: "jp-discovery-listview-message" }, "No entries"),
        pagination));
}
exports.ListView = ListView;
/**
 * The main view for the discovery extension.
 */
var ExtensionView = /** @class */ (function (_super) {
    __extends(ExtensionView, _super);
    function ExtensionView() {
        var _this = _super.call(this) || this;
        _this.model = new model_1.ListModel();
        _this.addClass('jp-discovery-view');
        return _this;
    }
    Object.defineProperty(ExtensionView.prototype, "inputNode", {
        /**
         * The search input node.
         */
        get: function () {
            return this.node.getElementsByClassName('jp-discovery-input')[0];
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Render the extension discovery view using the virtual DOM.
     */
    ExtensionView.prototype.render = function () {
        var _this = this;
        var model = this.model;
        var pages = Math.ceil(model.totalEntries / model.pagination);
        var elements = [
            React.createElement(SearchBar, { key: 'searchbar', placeholder: 'SEARCH' }),
        ];
        var content = [];
        if (!model.initialized) {
            model.initialize();
            content.push(React.createElement("div", { key: 'loading-placeholder', className: "jp-discovery-loader" }, "Updating extensions list"));
        }
        else if (!model.query && model.installed.length) {
            content.push(React.createElement("header", { key: 'installed-header' }, "Installed"), React.createElement(ListView, { key: 'installed', entries: model.installed, numPages: 1, onPage: function (value) { }, performAction: this.onAction.bind(this) }));
        }
        else if (!model.offline) {
            content.push(React.createElement("header", { key: 'installable-header' }, "Search results"), React.createElement(ListView, { key: 'installable', entries: model.searchResult, numPages: pages, onPage: function (value) { _this.onPage(value); }, performAction: this.onAction.bind(this) }));
        }
        else {
            content.push(React.createElement("div", { key: 'error-msg', className: "jp-discovery-error" },
                "Error searching for extensions",
                model.errorMessage ? ": " + model.errorMessage : '.'));
        }
        elements.push(React.createElement("div", { key: "content", className: "jp-discovery-content" }, content));
        return elements;
    };
    /**
     * Callback handler for the user specifies a new search query.
     *
     * @param value The new query.
     */
    ExtensionView.prototype.onSearch = function (value) {
        this.model.query = value;
    };
    /**
     * Callback handler for the user changes the page of the search result pagination.
     *
     * @param value The pagination page number.
     */
    ExtensionView.prototype.onPage = function (value) {
        this.model.page = value;
    };
    /**
     * Callback handler for when the user wants to perform an action on an extension.
     *
     * @param action The action to perform.
     * @param entry The entry to perform the action on.
     */
    ExtensionView.prototype.onAction = function (action, entry) {
        switch (action) {
            case 'install':
                return this.model.install(entry);
            case 'uninstall':
                return this.model.uninstall(entry);
            case 'enable':
                return this.model.enable(entry);
            case 'disable':
                return this.model.disable(entry);
            default:
                throw new Error("Invalid action: " + action);
        }
    };
    /**
   * Handle the DOM events for the command palette.
   *
   * @param event - The DOM event sent to the command palette.
   *
   * #### Notes
   * This method implements the DOM `EventListener` interface and is
   * called in response to events on the command palette's DOM node.
   * It should not be called directly by user code.
   */
    ExtensionView.prototype.handleEvent = function (event) {
        switch (event.type) {
            case 'input':
                this.onSearch(this.inputNode.value);
                break;
            case 'focus':
            case 'blur':
                this._toggleFocused();
                break;
        }
    };
    /**
     * A message handler invoked on a `'before-attach'` message.
     */
    ExtensionView.prototype.onBeforeAttach = function (msg) {
        this.node.addEventListener('input', this);
        this.node.addEventListener('focus', this, true);
        this.node.addEventListener('blur', this, true);
    };
    /**
     * A message handler invoked on an `'after-detach'` message.
     */
    ExtensionView.prototype.onAfterDetach = function (msg) {
        this.node.removeEventListener('input', this);
        this.node.removeEventListener('focus', this, true);
        this.node.removeEventListener('blur', this, true);
    };
    /**
     * A message handler invoked on an `'activate-request'` message.
     */
    ExtensionView.prototype.onActivateRequest = function (msg) {
        if (this.isAttached) {
            var input = this.inputNode;
            input.focus();
            input.select();
        }
    };
    /**
     * Toggle the focused modifier based on the input node focus state.
     */
    ExtensionView.prototype._toggleFocused = function () {
        var focused = document.activeElement === this.inputNode;
        this.toggleClass('p-mod-focused', focused);
    };
    return ExtensionView;
}(apputils_1.VDomRenderer));
exports.ExtensionView = ExtensionView;
//# sourceMappingURL=widget.js.map